<?php defined('SYSPATH') || die('No direct script access.');

class Model_Journal extends Model
{
    /**
     * @param $recordbook    int student's record book id
     * @param $discipline    int discipline id
     * @return Database_Result
     */
    public static function getAttendance($recordbook, $discipline) {
        $sql = 'CALL `get_attendance`(:recordbook, :discipline)';
        return DB::query(Database::SELECT, $sql)
            ->param(':recordbook', $recordbook)
            ->param(':discipline', $discipline)
            ->execute();
    }

    public static function SetAttendance($recordBookID, $lessonID, $attended) {
        $error = Model_Journal::SetAttendanceSQL($recordBookID, $lessonID, $attended);
        if (!is_null($error)) return $error;
    }

   public static function SetAttendanceSQL($recordBookID, $lessonID, $attended) {
        $sql = 'SELECT * FROM set_student_attendance(:lesson, :recordBook, :attended) AS "Num"';
        $dbAnswer = DB::query(Database::SELECT, $sql)
            ->parameters([
                ':lesson'     => $lessonID,
                ':recordBook' => $recordBookID,
                ':attended'   => (int) $attended
            ])->execute();

        $result = $dbAnswer->get('Num');
        if ($result < 0) {
            return ['code'=>400, 'message'=>'Ошибка приложения'];
        }
        return null;
    }

    public static function getAttendanceForDiscipline(Model_Discipline $discipline) {
        $sql = 'SELECT * FROM get_attendance_for_discipline(:id)';
        $result = DB::query(Database::SELECT, $sql)
            ->param(':id', $discipline->ID)->execute();

        $table = [];
        foreach ($result as $row) {
            $recordBook = $row['RecordBookID'];
            $lesson = $row['LessonID'];
            $table[$recordBook][$lesson] = $row['Attendance'];
        }

        return $table;
    }

    public static function getAttendanceForStudentsGroup($discipline, $group) {
        $sql = 'SELECT * FROM get_attendance_for_group(:discipline, :group)';
        return DB::query(Database::SELECT, $sql)
            ->parameters([
                ':discipline' => $discipline,
                ':group'      => $group,
            ])->execute();
    }

    public static function getAttendanceForStudent($discipline, $student) {
        $sql = 'SELECT * FROM  get_attendance_for_student(:discipline, :student)';
        return DB::query(Database::SELECT, $sql)
            ->parameters([
                ':discipline' => $discipline,
                ':student'      => $student,
            ])->execute();
    }

    public static function getLessonsForDiscipline($discipline) {
        $sql = 'SELECT * FROM get_lessons_for_discipline(:discipline)';
        return DB::query(Database::SELECT, $sql)
            ->parameters([
                ':discipline' => $discipline
            ])->execute();
    }

    public static function getHistory($discipline) {
        $sql = 'SELECT * FROM get_attendance_history(:discipline)';
        return DB::query(Database::SELECT, $sql)
            ->param(':discipline', $discipline)
            ->execute();
    }

    public static function createLesson($discipline, $date, $type) {
        $sql = 'SELECT * FROM create_lesson(:discipline, :date, :type) AS "Num"';
        $dbAnswer = DB::query(Database::SELECT, $sql)
            ->param(':discipline', $discipline)
            ->param(':date', $date)
            ->param(':type', $type)
            ->execute();

        $result = $dbAnswer->get('Num');
        if ($result < 0) {
            return ['code'=>400, 'message'=>'Ошибка приложения'];
        } else {
            return $result;
        }
    }

    public static function deleteLesson($id) {
        $sql = 'SELECT * FROM delete_lesson(:id) AS "Num"';
        $dbAnswer = DB::query(Database::SELECT, $sql)
            ->param(':id', $id)
            ->execute();

        $result = $dbAnswer->get('Num');
        if ($result < 0) {
            return ['code'=>400, 'message'=>'Ошибка приложения'];
        } else {
            return $result;
        }
    }

    public static function setLessonDate($lesson, $date) {
        $sql = 'SELECT * FROM set_lesson_date(:lesson, :date) AS "Num"';
        $dbAnswer = DB::query(Database::SELECT, $sql)
            ->param(':lesson', $lesson)
            ->param(':date', $date)
            ->execute();

        $result = $dbAnswer->get('Num');
        if ($result < 0) {
            return ['code'=>400, 'message'=>'Ошибка приложения'];
        }
        return null;
    }

    public static function getLessonTypes() {
        $sql = 'SELECT * FROM get_lesson_types()';
        return DB::query(Database::SELECT, $sql)->execute();
    }
};